// -*- C++ -*-
/*!
 * @file RtcBase.h
 *
 * @if jp
 * @brief RT component 쥯饹
 * @else
 * @brief RT component base class
 * @endif
 *
 * @date $Date$
 *
 * Copyright (c) 2003 Noriaki Ando <n-ando@aist.go.jp>
 *          Task-intelligence Research Group,
 *          Intelligent System Research Institute,
 *          National Institute of Industrial Science (AIST), Japan
 *          All rights reserved.
 *
 * $Id$
 */

#ifndef RtcBase_h
#define RtcBase_h

// CORBA header include
#include "rtm/RTC.h"

// STL includes
#include <list>
#include <string>
#include <algorithm>
#include <functional>

// local includes 
#include "rtm/idl/RTCBaseSkel.h"
#include "rtm/idl/RTCBaseStub.h"
#include "rtm/RtcOutPort.h"
#include "rtm/RtcModuleProfile.h"
#include "rtm/RtcSystemLogger.h"

#include "rtm/Rtc_Mutex.h"
#include "rtm/Rtc_Task.h"
#include "rtm/Rtc_Util.h"


#define USLEEP(x) usleep(x);

namespace RTM {
  // namespace
  using namespace std;
  
  
  class InPortBase;
  class OutPortBase;
  class RtcManager;
  
  static char* StateString[] =
	{
	  "RTC_UNKNOWN",
	  "RTC_BORN",
	  "RTC_INITIALIZING",
	  "RTC_READY",
	  "RTC_STARTING",
	  "RTC_ACTIVE",
	  "RTC_STOPPING",
	  "RTC_ABORTING",
	  "RTC_ERROR",
	  "RTC_FATAL_ERROR",
	  "RTC_EXITING",
	  "UNKNOWN"
	};

  /*!
   * @if jp
   * @brief RTComponent ַ
   * @else
   * @brief RTComponent state re-declaration
   * @endif
   */
  typedef RTM::RTComponent::ComponentState ComponentState;
  
  const ComponentState RTC_UNKNOWN      = RTM::RTComponent::RTC_UNKNOWN;
  const ComponentState RTC_BORN         = RTM::RTComponent::RTC_BORN;
  const ComponentState RTC_INITIALIZING = RTM::RTComponent::RTC_INITIALIZING;
  const ComponentState RTC_READY        = RTM::RTComponent::RTC_READY;
  const ComponentState RTC_STARTING     = RTM::RTComponent::RTC_STARTING;
  const ComponentState RTC_ACTIVE       = RTM::RTComponent::RTC_ACTIVE;
  const ComponentState RTC_STOPPING     = RTM::RTComponent::RTC_STOPPING;
  const ComponentState RTC_ABORTING     = RTM::RTComponent::RTC_ABORTING;
  const ComponentState RTC_ERROR        = RTM::RTComponent::RTC_ERROR;
  const ComponentState RTC_FATAL_ERROR  = RTM::RTComponent::RTC_FATAL_ERROR;
  const ComponentState RTC_EXITING      = RTM::RTComponent::RTC_EXITING;
  

  typedef enum NamingPolicy {
	UNKNOWN = 0,
	LONGNAME_ENABLE = 1,
	ALIAS_ENABLE = 2
  };

  /*!
   * @if jp
   *
   * @class RtcBase
   *
   * @brief RTComponent 쥯饹
   *
   * RTComponent ȯԤϿ˺륳ݡͥȤΥ饹򤳤RtcBase
   * ֥饹ȤʤФʤʤ
   * ˺륳ݡͥȤˤơɬפʥƥӥƥξ֤б
   * ᥽å rtc_active_do() ŬС饤ɤơƾ֤ˤƹԤ
   * Υ᥽åɤ˵Ҥ롣
   * rtc_xxx_entry(), rtc_xxx_exit() ǤϤξ֤ȤȽФȤ˰٤
   * Υ᥽åɤ¹Ԥ졢rtc_xxx_do() ǤϤξ֤ˤ桢Υ᥽
   * åɤ¹Ԥ롣
   * ƥ᥽å rtc_xxx_[entry|do|exit]() Ǥͤ RTM_OK, RTM_ERR, 
   * RTM_WARNING, RTM_FATAL_ERR Τ줫֤
   * RTM_ERR ֤ ERROR ֤ˡRTM_FATAL_ERR ֤ FATAL_ERROR ֤
   * ܹԤ롣¾ξܤˤĤƤϥޥ˥奢򻲾ȡ
   * 
   * ˺륳ݡͥȤΥ饹Ǥϡ InPort , OutPort Ȥ
   * Хɤ InPort , OutPort ηбѿ롣
   * ݡͥȤΥ饹Υ󥹥ȥ饯ǤϡѿҤѤ
   * 󥹥ȥ饯 registerInPort , registerOutPort Ѥ
   * 줾 InPort , OutPort ȤϿɬפ롣
   *
   * @else
   *
   * @class RtcBase
   *
   * @brief RT component base class.
   *
   * RT-component developer should derive this base class to create own RT
   * component class.
   * Procedure is as follows.
   *
   * In your RT-conponent class, you have to define input variable and output
   * variable. Input channel and output channel also should be defined as 
   * InPort<T> objects or OutPort<T> objects. These variables and channel
   * objects should be bound each other in the constructor.
   *
   * In the constructor of your component class, input channel objects and 
   * output channel objects have to make registration as CORBA objects using
   * registerInPort , registerOutPort method.
   *
   * @endif
   */
  class RtcBase  
	: public virtual POA_RTM::RTCBase, 
      public virtual PortableServer::RefCountServantBase,
      public RTC_Task
//      public ACE_Task<ACE_MT_SYNCH>
  {
  public:

	/*!
	 * @if jp
	 * @brief RtcBase 饹󥹥ȥ饯
	 *
	 * RTComponent Хȼδ쥯饹 RtcBase Υ󥹥ȥ饯
	 * Υ󥹥ȥ饯Ѥ硢桼ORBؤΥݥ󥿤POAΥݥ
	 * 뤤ϡޥ͡ؤΥݥ󥿤ǥåȤɬפ롣
	 *
	 * @else
	 *
	 * @brief A constructor of RtcBase class.
	 *
	 * Constructor of RtcBase which is RTComponent servant implementation
	 * base classes without arguments.
	 * If use this constructor, user have to set the ORB pointer and a POA
	 * pointer or a pointer to RtcManager.
	 *
	 * @endif
	 */
	RtcBase();
	
	/*!
	 * @if jp
	 * @brief RtcBase 饹󥹥ȥ饯
	 *
	 * RTComponent Хȼδ쥯饹 RtcBase Υ󥹥ȥ饯
	 *
	 * ORB ؤΥݥ󥿤POAؤΥݥ󥿤ǤƤˤϡ
	 * 󥹥ȥ饯Ѥ뤳Ȥ롣
	 *
	 * @param orb ORB ؤΥݥ
	 * @param poa POA ؤΥݥ
	 *
	 * @else
	 *
	 * @brief A constructor of RtcBase class.
	 *
	 * If the ORB pointer and a POA pointer are already available, user can use
	 * this constructor.
	 *
	 * @param orb pointer to ORB
	 * @param poa pointer to POA
	 *
	 * @endif
	 */
	RtcBase(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa);

	/*!
	 * @if jp
	 *
	 * @brief RtcBase 饹󥹥ȥ饯
	 *
	 * ̾ϥޥ̤͡ƥݡͥȥ饹򥤥󥹥󥹲
	 * Υ󥹥ȥ饯Ѥ뤳Ȥ侩롣
	 *
	 * @param manager ݡͥȥޥ͡ RtcManager ؤΥݥ
	 *
	 * @else
	 *
	 * @brief A constructor.
	 *
	 * @param manager pointer to component manager: RtcManager
	 * @endif
	 */
	RtcBase(RtcManager* manager);
	
	/*!
	 * @if jp
	 * 
	 * @brief RtcBase 饹ǥȥ饯
	 *
	 * @else
	 *
	 * @brief A destructor of RtcBase class.
	 *
	 * @endif
	 */
	virtual ~RtcBase();
	
	//============================================================
	//
	// <<< CORBA interfaces >>>
	//
	//============================================================
	
	//============================================================
	// State implementation functions 
	//============================================================
	/*
	 * 
	 *                                   __                      __
	 *                         _________V_ \          __________V_ \
	 *                        |           |/  (any)->|            |/
	 *                        |  Active   |          | FatalError |
	 *                        |___________|--,       |____________|
	 *                            A    |  ___|______      |
	 *                      ______|___ | |          |     |
	 *                     |          || | Aborting |     |
	 *                     | Starting || |__________|     |
	 *                     |__________||     |   __       |
	 *             ____________   |    |   __V__V_ \   ___V_____
	 *   _____    |            |<-+----+--|       |/  |         |    ___
	 *  (Start)-->|Initializing|  |    |  | Error |-->| Exiting |-->(End)
	 *   ~~~~~    |____________|--+----+->|_______|   |_________|    ~~~
	 *                A  |        | ___|______  A         A
	 *                |  |        ||          | |         |
	 *                |  |        || Stopping | |         |
	 *                |  |        ||__________| |         |
	 *                |  |        |    |        |         |
	 *                |  |     ___|____V__      |         |
	 *                |  `--->|           |-----'         |
	 *                `-------|   Ready   |               |
	 *                       /|___________|---------------'
	 *                       \__A
	 *
	 */
	
	//------------------------------------------------------------
	// Transition operation of component activity state
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * 
	 * @brief [CORBA interface] ݡͥȤΥƥֲ
	 *
	 * ݡͥȤξ֤ READY  ACTIVE ܤ롣
	 * Υڥ졼ȯԤȤݡͥȤ READY ֤Ǥʤ
	 * ʤʤ¾ξ֤ξˤ IllegalTransition 㳰ȯ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Activate this components' activity
	 * 
	 * @endif
	 */
	virtual RtmRes rtc_start()
	  throw (CORBA::SystemException, RTM::RTComponent::IllegalTransition);
	
	/*!
	 * @if jp
	 * 
	 * @brief [CORBA interface] ݡͥȤ󥢥ƥֲ
	 *
	 * ݡͥȤξ֤ ACTIVE  READY ܤ롣
	 * Υڥ졼ȯԤȤݡͥȤ ACTIVE ֤Ǥʤ
	 * ʤʤ¾ξ֤ξˤ IllegalTransition 㳰ȯ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Stop this component's activity
	 * 
	 * @endif
	 */
	virtual RtmRes rtc_stop()
	  throw (CORBA::SystemException, RTM::RTComponent::IllegalTransition);
	
	/*!
	 * @if jp
	 * 
	 * @brief [CORBA interface] ݡͥȤΥꥻå
	 *
	 * ݡͥȤξ֤ ERROR  INITIALIZE ܤ롣
	 * INITIALIZE 奨顼ʤФ READY ֤ܤ롣
	 * Υڥ졼ȯԤȤݡͥȤ ERROR ֤Ǥʤ
	 * ʤʤ¾ξ֤ξˤ IllegalTransition 㳰ȯ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Go to init() state from error() state
	 * 
	 * @endif
	 */
	virtual RtmRes rtc_reset()
	  throw (CORBA::SystemException, RTM::RTComponent::IllegalTransition);
	
	/*!
	 * @if jp
	 * 
	 * @brief [CORBA interface] ݡͥȤΥꥻå
	 *
	 * ݡͥȤξ֤ EXITING ܤ롣
	 * EXITING ֤ܤݡͥȤ٤뤳Ȥʤλ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Go to term() state from anywhere
	 * 
	 * @endif
	 */
	virtual RtmRes rtc_exit()
	  throw (CORBA::SystemException, RTM::RTComponent::IllegalTransition);
	
	/*!
	 * @if jp
	 * 
	 * @brief [CORBA interface] ݡͥȤζλ
	 *
	 * FATAL_ERROR ֤ΥݡͥȤ EXITING ܤ롣
	 * EXITING ֤ܤݡͥȤ٤뤳Ȥʤλ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Go to exiting() state from active() state 
	 * 
	 * @endif
	 */
	virtual RtmRes rtc_kill();
	
	//------------------------------------------------------------
	// Main activity
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * 
	 * @brief [CORBA interface] ᥤ󥢥ƥӥƥΥ᥽å
	 *
	 * ݡͥȤΥƥӥƥΤϤΥ᥽åɤ¹Ԥ뤳Ȥ
	 * 롣ñΤΥݡͥȤǤŪʥåɤˤꤳΥ᥽
	 * åɤƽФ뤳ȤǽԤäƤ롣
	 * åɤߤ餳Υڥ졼ƤӽФȤˤꡢǤ
	 * դΥߥ󥰤ǥƥӥƥ¹Ԥ뤳Ȥ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Main activity function. 
	 *
	 * Called from activity thread periodically.
	 * 
	 * @endif
	 */
	virtual RtmRes rtc_worker();
	
	
	// ** Steady state **
	//------------------------------------------------------------
	// Ready state methods
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: ready() ᥽å
	 *
	 * READY ֤ؿȤ1٤ƤӽФ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: ready() method.
	 *
	 * This method will be called once when component enter
	 * to Ready state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_ready_entry()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] do: ready() ᥽å.
	 *
	 * READY ֤ˤּ¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] do: ready() method.
	 *
	 * This method will be called repeatedly when component state is Ready.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_ready_do()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] exit: ready() ᥽å.
	 *
	 * READY ֤ФȤ1٤¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] exit: ready() method.
	 *
	 * This method will be called once when component exit from Ready state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_ready_exit()
	{USLEEP(1000); return RTM_OK;};
	
	
	//------------------------------------------------------------
	// Active state methods
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: active() ᥽å.
	 *
	 * ACTIVE ֤˿Ȥ1٤¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: active() method.
	 *
	 * This method will be called once when component enter to Active state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_active_entry()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] do: active() ᥽å.
	 *
	 * ACTIVE ֤˵ּ¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] do: active() method.
	 *
	 * This method will be called repeatedly when component state is Active.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_active_do()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] exit: active() ᥽å.
	 *
	 * ACTIVE ֤ФȤ1٤¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] exit: active() method.
	 *
	 * This method will be called once when component exit from Active state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_active_exit()
	{USLEEP(1000); return RTM_OK;};
	
	
	//------------------------------------------------------------
	// Error state methods
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: error() ᥽å
	 *
	 * ERROR ֤ؿȤ1٤ƤӽФ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: error() method.
	 *
	 * This method will be called once when component enter to Error state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_error_entry()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] do: error() ᥽å.
	 *
	 * ERROR ֤ˤּ¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] do: ready() method.
	 *
	 * This method will be called repeatedly when component state is Error.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_error_do()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] exit: error() ᥽å.
	 *
	 * ERROR ֤ФȤ1٤¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] exit: error() method.
	 *
	 * This method will be called once when component exit from Error state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_error_exit()
	{USLEEP(1000); return RTM_OK;};
	
	
	//------------------------------------------------------------
	// FatalError state methods
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: fatal() ᥽å
	 *
	 * FATAL_ERROR ֤ؿȤ1٤ƤӽФ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: fatal() method.
	 *
	 * This method will be called once when component enter to FATAL_ERROR
	 * state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_fatal_entry()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] do: fatal() ᥽å.
	 *
	 * FATAL_ERROR ֤ˤּ¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] do: fatal() method.
	 *
	 * This method will be called repeatedly when component state is
	 * FATAL_ERROR state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_fatal_do()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] exit: fatal() ᥽å.
	 *
	 * READY ֤ФȤ1٤¹Ԥ᥽åɡ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] exit: fatal() method.
	 *
	 * This method will be called once when component exit from FATAL_ERROR
	 * state.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_fatal_exit()
	{USLEEP(1000); return RTM_OK;};
	//------------------------------------------------------------
	
	// ** Transient state **
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: init() ᥽å
	 *
	 * INITIALIZING ֤ؿȤ1٤ƤӽФ᥽åɡ
	 * 顼ʤ READY ֤ܤ롣
	 * ͤ RTM_ERR  ERROR ֤ء RTM_FATALERR  FATAL_ERROR ֤
	 * ܤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: ready() method.
	 *
	 * This method will be called and move to Ready/Error state immediately.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_init_entry()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: starting() ᥽å
	 *
	 * STARTING ֤ؿȤ1٤ƤӽФ᥽åɡ
	 * 顼ʤ ACTIVE ֤ܤ롣
	 * ͤ RTM_ERR  ERROR ֤ء RTM_FATALERR  FATAL_ERROR ֤
	 * ܤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: starting() method.
	 *
	 * This method will be called and move to Active state immediately.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_starting_entry()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: stopping() ᥽å
	 *
	 * STOPPING ֤ؿȤ1٤ƤӽФ᥽åɡ
	 * 顼ʤ READY ֤ܤ롣
	 * ͤ RTM_ERR  ERROR ֤ء RTM_FATALERR  FATAL_ERROR ֤
	 * ܤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: stopping() method.
	 *
	 * This method will be called and move to Ready state immediately.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_stopping_entry()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: aborting() ᥽å
	 *
	 * ABORTING ֤ؿȤ1٤ƤӽФ᥽åɡ
	 * 顼ʤ READY ֤ܤ롣
	 * ͤ RTM_ERR  ERROR ֤ء RTM_FATALERR  FATAL_ERROR ֤
	 * ܤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: aborting() method.
	 *
	 * This method will be called and move to Error state immediately.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_aborting_entry()
	{USLEEP(1000); return RTM_OK;};
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] entry: exiting() ᥽å
	 *
	 * EXITING ֤ؿȤ1٤ƤӽФ᥽åɡ
	 * 顼ʤ READY ֤ܤ롣
	 * ͤ RTM_ERR  ERROR ֤ء RTM_FATALERR  FATAL_ERROR ֤
	 * ܤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] entry: exiting() method.
	 *
	 * This method will be called and terminate component.
	 *
	 * @endif
	 */
	virtual RtmRes rtc_exiting_entry()
	{USLEEP(1000); return RTM_OK;};
	
	
	//------------------------------------------------------------
	// Status handling
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * @brief [CORBA interface] ƥӥƥơOutPortμ
	 *
	 * ƥӥƥơOutPortΥ֥ȥե󥹤롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get activity status OutPort
	 *
	 * @endif
	 */
	virtual OutPort_ptr rtc_state();
	
	//------------------------------------------------------------
	// Get inports and outport object reference
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * @brief [CORBA interface] InPortList μ
	 *
	 * InPort Υ֥ȥե󥹤ΥꥹȤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Return input port object reference list
	 *
	 * @endif
	 */
	virtual InPortList* inports() ;

	/*!
	 * @if jp
	 * @brief [CORBA interface] InPort μ
	 *
	 * InPort Υ֥ȥե󥹤롣
	 *
	 * @param name InPort ̾
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Return input port object reference
	 *
	 * @param name InPort name
	 *
	 * @endif
	 */
	virtual InPort_ptr get_inport(const char* name)
	  throw (CORBA::SystemException, RTM::RTComponent::NoSuchName); 
	
	/*!
	 * @if jp
	 * @brief [CORBA interface] OutPortList μ
	 *
	 * OutPort Υ֥ȥե󥹤ΥꥹȤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Return output port object reference list
	 *
	 * @endif
	 */
	virtual OutPortList* outports() ;

	/*!
	 * @if jp
	 * @brief [CORBA interface] OutPort μ
	 *
	 * OutPort Υ֥ȥե󥹤롣
	 *
	 * @param name OutPort ̾
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Return output port object reference
	 *
	 * @param name OutPort name
	 *
	 * @endif
	 */
	virtual OutPort_ptr get_outport(const char* name)
	  throw (CORBA::SystemException, RTM::RTComponent::NoSuchName);
	
	//------------------------------------------------------------
	// [CORBA interface] Getting component property
	//------------------------------------------------------------
	//  CORBA::Any* get_property(const char* name) {return NULL;};
	//  char* get_properties_xml() {return NULL;};
	//  virtual char* instance_name() {return "RtcBase0";}
	//  virtual char* class_name() {return "RtcBase";}

	/*!
	 * @if jp
	 * @brief [CORBA interface] instance_id μ
	 *
	 * ݡͥȤΥ󥹥ID롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] get instance_id
	 *
	 * @endif
	 */
	virtual char* instance_id();

	/*!
	 * @if jp
	 * @brief [CORBA interface] implementation_id μ
	 *
	 * ݡͥȤΥץơID롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] get implementation_id
	 *
	 * @endif
	 */
    virtual char* implementation_id();

	/*!
	 * @if jp
	 * @brief [CORBA interface] description μ
	 *
	 * ݡͥȤγ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] get description
	 *
	 * @endif
	 */
    virtual char* description();

	/*!
	 * @if jp
	 * @brief [CORBA interface] version μ
	 *
	 * ݡͥȤΥС롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] get version
	 *
	 * @endif
	 */
    virtual char* version();

	/*!
	 * @if jp
	 * @brief [CORBA interface] maker μ
	 *
	 * ݡͥȤκԤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] get maker
	 *
	 * @endif
	 */
    virtual char* maker();

	/*!
	 * @if jp
	 * @brief [CORBA interface] category μ
	 *
	 * ݡͥȤΥƥ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] get category
	 *
	 * @endif
	 */
    virtual char* category();

	/*!
	 * @if jp
	 * @brief [CORBA interface] profile μ
	 *
	 * ݡͥȤΥץե롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] get profile
	 *
	 * @endif
	 */
	virtual RTCProfile* profile() ;
	//	virtual RTCConfiguration* configuration() ;

	
	
	//------------------------------------------------------------
	// [CORBA interface] composit component interfaces
	//
	// These interfaces are methods for RTCompositComponet
	// Almost all methods do nothing. Only rtc_components and 
	// rtc_get_componet methods return this component itself.
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * @brief [CORBA interface] ƥӥƥåɤΥ
	 *
	 * ݡͥȥƥӥƥåɤ򥹥Ȥ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Start activity internal thread
	 *
	 * @endif
	 */
	virtual RtmRes rtc_start_thread();

	/*!
	 * @if jp
	 * @brief [CORBA interface] ƥӥƥåɤΥڥ
	 *
	 * ݡͥȥƥӥƥåɤ򥵥ڥɤ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Suspend activity internal thread
	 *
	 * @endif
	 */
	virtual RtmRes rtc_suspend_thread();

	/*!
	 * @if jp
	 * @brief [CORBA interface] ƥӥƥåɤΥȥå
	 *
	 * ݡͥȥƥӥƥåɤ򥹥ȥåפ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Stop activity internal thread
	 *
	 * @endif
	 */
	virtual RtmRes rtc_stop_thread();

	/*!
	 * @if jp
	 * @brief [CORBA interface] ƥݡͥȤ򥻥åȤ
	 *
	 * ƥݡͥȤΥ֥ȥե󥹤򥻥åȤ롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Set parent component
	 *
	 * @endif
	 */
	virtual RtmRes rtc_set_parent(RTCBase_ptr comp);

	/*!
	 * @if jp
	 * @brief [CORBA interface] ҥݡͥȤɲä
	 *
	 * ҥݡͥȤΥ֥ȥե󥹤򥻥åȤ롣
	 * ñΥݡͥȤˤƤΥ᥽åɤ̤ѤΤᡢRTM_ERR ֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Set child component
	 *
	 * @endif
	 */
	virtual RtmRes rtc_add_component(RTCBase_ptr comp)
	{return RTM_ERR;};

	/*!
	 * @if jp
	 * @brief [CORBA interface] ҥݡͥȤ
	 *
	 * ҥݡͥȤΥ֥ȥե󥹤롣
	 * ñΥݡͥȤˤƤΥ᥽åɤ̤ѤΤᡢRTM_ERR ֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Delete child component
	 *
	 * @endif
	 */
	virtual RtmRes rtc_delete_component(RTCBase_ptr comp)
	{return RTM_ERR;} ;

	/*!
	 * @if jp
	 * @brief [CORBA interface] ҥݡͥȤνؤ
	 *
	 * 2ĤλҥݡͥȤ򥪥֥ȥե󥹤Ѥƽؤ
	 * 롣
	 * ñΥݡͥȤˤƤΥ᥽åɤ̤ѤΤᡢRTM_ERR ֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Replace two child components
	 *
	 * @endif
	 */
	virtual RtmRes rtc_replace_component(RTCBase_ptr comp1, RTCBase_ptr comp2)
	{return RTM_ERR;};

	/*!
	 * @if jp
	 * @brief [CORBA interface] ҥݡͥȤνؤ
	 *
	 * 2ĤλҥݡͥȤν򥳥ݡͥ̾ꤷؤ롣
	 * ñΥݡͥȤˤƤΥ᥽åɤ̤ѤΤᡢRTM_ERR ֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Replace two child components
	 *
	 * @endif
	 */
	virtual RtmRes rtc_replace_component_by_name(const char* name1, const char* name2)
	{return RTM_ERR;};

	/*!
	 * @if jp
	 * @brief [CORBA interface] ҥݡͥȤꥹȤȤƼ롣
	 *
	 * ҥݡͥȤΥꥹȤ롣
	 * ñΥݡͥȤˤƤϼʬȥ֥ȥե󥹤֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RTCBaseList* rtc_components()
	{
	  RTCBaseList_var myself = new RTCBaseList();
	  myself->length(1);

#ifdef ORB_IS_TAO
	  (*myself)[0] = RTM::RTCBase::_narrow(m_pPOA->servant_to_reference(this));
#else
	  myself[0] = RTM::RTCBase::_narrow(m_pPOA->servant_to_reference(this));
#endif
	  return myself._retn();
	};

	/*!
	 * @if jp
	 * @brief [CORBA interface] ҥݡͥȤ̾ꤷƼ
	 *
	 * ҥݡͥȤ̾ꤷƤΥ֥ȥե󥹤롣
	 * ñΥݡͥȤˤƤϼʬȥ֥ȥե󥹤֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RTCBase_ptr rtc_get_component(const char* name)
	{
	  return dynamic_cast<RTCBase_ptr>(m_pPOA->servant_to_reference(this));
	};

	/*!
	 * @if jp
	 * @brief [CORBA interface] InPort򥢥å
	 *
	 * ҥݡͥȤInPort򤳤ΥݡͥȤInPort˥å롣
	 * ñΥݡͥȤˤƤϼʬȥ֥ȥե󥹤֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RtmRes rtc_attach_inport(InPort_ptr in_port)
	{return RTM_ERR;};

	/*!
	 * @if jp
	 * @brief [CORBA interface] InPort򥢥å
	 *
	 * ҥݡͥȤInPort̾ꤷƥݡͥȤInPort˥å
	 * 롣
	 * ñΥݡͥȤˤƤϼʬȥ֥ȥե󥹤֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RtmRes rtc_attach_inport_by_name(RTCBase_ptr comp, const char* name)
	{return RTM_ERR;};


	/*!
	 * @if jp
	 * @brief [CORBA interface] InPortǥå
	 *
	 * ҥݡͥȤInPort򤳤ΥݡͥȤInPortǥå롣
	 * ñΥݡͥȤˤƤ RTM_ERR ֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RtmRes rtc_detatch_inport(InPort_ptr in_port)
	{return RTM_ERR;};

	/*!
	 * @if jp
	 * @brief [CORBA interface] InPortǥå
	 *
	 * ҥݡͥȤInPort̾ꤷƤΥݡͥȤInPort
	 * ǥå롣
	 * ñΥݡͥȤˤƤ RTM_ERR ֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RtmRes rtc_detatch_inport_by_name(const char* name)
	{return RTM_ERR;};

	/*!
	 * @if jp
	 * @brief [CORBA interface] OutPort򥢥å
	 *
	 * ҥݡͥȤOutPort򤳤ΥݡͥȤOutPort˥å롣
	 * ñΥݡͥȤˤƤϼʬȥ֥ȥե󥹤֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RtmRes rtc_attach_outport(OutPort_ptr out_port)
	{return RTM_ERR;} ;

	/*!
	 * @if jp
	 * @brief [CORBA interface] OutPort򥢥å
	 *
	 * ҥݡͥȤOutPort̾ꤷƥݡͥȤInPort˥å
	 * 롣
	 * ñΥݡͥȤˤƤϼʬȥ֥ȥե󥹤֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RtmRes rtc_attach_outport_by_name(RTCBase_ptr comp,
											  const char* name)
	{return RTM_ERR;};

	/*!
	 * @if jp
	 * @brief [CORBA interface] OutPortǥå
	 *
	 * ҥݡͥȤOutPort򤳤ΥݡͥȤOutPortǥå롣
	 * ñΥݡͥȤˤƤ RTM_ERR ֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RtmRes rtc_detatch_outport(OutPort_ptr out_port)
	{return RTM_ERR;}

	/*!
	 * @if jp
	 * @brief [CORBA interface] OutPortǥå
	 *
	 * ҥݡͥȤOutPort̾ꤷƤΥݡͥȤOutPort
	 * ǥå롣
	 * ñΥݡͥȤˤƤ RTM_ERR ֤
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get child components list
	 *
	 * @endif
	 */
	virtual RtmRes rtc_detatch_outport_by_name(const char* name)
	 {return RTM_ERR;};
	
	// end of CORBA interface definition
	//============================================================
	
	
	
	//============================================================
	//
	// <<< Local interfaces >>>
	//
	//============================================================
	
	/*!
	 * @if jp
	 * @brief ORB, POA Υݥ󥿤Ϳƽ
	 *
	 * ݡͥȤORBPOAΥݥ󥿤ͿƥݡͥȤ롣
	 *
	 * @param orb ORB ؤΥݥ
	 * @param poa POA ؤΥݥ
	 *
	 * @else
	 *
	 * @brief Initialize this component with ORB and POA pointer
	 *
	 * @param orb pointer to ORB
	 * @param poa pointer to POA
	 *
	 * @endif
	 */
	virtual void init_orb(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa);
	
	/*!
	 * @if jp
	 * @brief ݡͥȤΥƥӥƥåɤ
	 *
	 * ݡͥȤƥӥƥåɤư롣
	 *  ACE_Task ӥ饹᥽åɤΥС饤ɡ
	 *
	 * @param args ̾0
	 *
	 * @else
	 *
	 * @brief Create internal activity thread
	 *
	 * Create internal activity thread. This is ACE_Task class method over ride.
	 *
	 * @param args usually give 0
	 *
	 * @endif
	 */	
	virtual int open(void *args);
	
	/*!
	 * @if jp
	 * @brief ݡͥȤΥƥӥƥåɴؿ
	 *
	 * ݡͥȤƥӥƥåɤμ¹Դؿ
	 *  ACE_Task ӥ饹᥽åɤΥС饤ɡ
	 *
	 * @else
	 *
	 * @brief Create internal activity thread
	 *
	 * Run by a daemon thread to handle deferred processing.
	 * ACE_Task class method override.
	 *
	 * @endif
	 */	
	virtual int svc(void);
	

	/*!
	 * @if jp
	 * @brief ݡͥȤΥƥӥƥåɽλؿ
	 *
	 * ݡͥȤƥӥƥåɽλ˸ƤФ롣
	 * ݡͥȥ֥Ȥ󥢥ƥֲޥ͡ؤΤԤ
	 *  ACE_Task ӥ饹᥽åɤΥС饤ɡ
	 *
	 * @else
	 *
	 * @brief Close activity thread
	 *
	 * close() method is called when activity thread svc() is returned.
	 * This method deactivate this object and notify it to manager.
	 * ACE_Task class method override.
	 *
	 * @endif
	 */	
	virtual int close(unsigned long flags);

	/*!
	 * @if jp
	 *
	 * @brief ݡͥȥơȼ
	 *
	 * ݡͥȤθߤξ֤롣
	 *
	 * @else
	 *
	 * @brief Get current component state
	 *
	 * @endif
	 */
	virtual RTM::RTComponent::ComponentState getState();

	//============================================================
	// Component Property handling methods
	//============================================================
	// @brief Get component class name.

	/*!
	 * @if jp
	 *
	 * @brief ModuleProfile ν
	 *
	 * RtcModuleProfile 饹Υ󥹥󥹤ϤơݡͥȤ
	 * ץե롣
	 *
	 * @param RtcModuleProfSpec ⥸塼ץե
	 *
	 * @else
	 *
	 * @brief Initialize RtcModuleProfile
	 *
	 * @param RtcModuleProfile module profile
	 *
	 * @endif
	 */
	virtual void initModuleProfile(RtcModuleProfile prof);

	/*!
	 * @if jp
	 *
	 * @brief ModuleProfile μ
	 *
	 * ModuleProfile 롣
	 *
	 * @else
	 *
	 * @brief Get RtcModuleProfile
	 *
	 * Get RtcModuleProfile
	 *
	 * @endif
	 */
	virtual RtcModuleProfile& getModuleProfile()
	{
	  return m_Profile;
	}
	/*
	virtual RtcComponentProfile& getProfile()
	{
	  return m_Profile;
	}
	*/
	/*
	virtual RtcConfiguration& getConfiguration()
	{
	  return m_Configuration;
	}
	*/
	/*!------------------------------------------------------------
	 * Set component name
	 *------------------------------------------------------------*/  
	/*!
	 * @if jp
	 *
	 * @brief ݡͥ̾򥻥åȤ
	 *
	 * ݡͥȤΥ⥸塼̾˰Ϳղä
	 * ݡ֤ͥ̾
	 * 桼ϤΥ᥽åɤ򥪡С饤ɤơݡͥ̾̿̾
	 * ˡ򥫥ޥ뤳Ȥ롣
	 *
	 * @param num ݡͥȤΥ󥹥ֹ
	 *
	 * @else
	 *
	 * @brief Set component name
	 *
	 * @param num instance number of this component
	 *
	 * @endif
	 */
	virtual string setComponentName(int num);

	//============================================================
	// Input port handling
	//============================================================
	/*!
	 * @if jp
	 *
	 * @brief InPort Ͽ
	 *
	 * InPort 򥳥ݡͥȤϿ롣Ͽ줿InPortϳ
	 * 褦ˤʤ롣
	 *
	 * @param in_ch InPort object of InPort<T>
	 *
	 * @else
	 *
	 * @brief Registration of input channel.
	 *
	 * Duplication of a name of input channel is not allowed.
	 * "false" will be returned when there is duplication of a name.
	 *
	 * @param in_ch Input channel object of InPort<T>
	 * @return true/false: "false" will be returned if registration goes wrong.
	 *
	 * @endif
	 */
	bool registerInPort(InPortBase& in_ch);

	/*!
	 * @if jp
	 *
	 * @brief InPort Ͽ
	 *
	 * InPort 򥳥ݡͥȤϿ롣Ͽ줿InPortϳ
	 * 褦ˤʤ롣
	 *
	 * @param in_ch InPort object of InPort<T>
	 *
	 * @else
	 *
	 * @brief Registration of input channel.
	 *
	 * Duplication of a name of input channel is not allowed.
	 * "false" will be returned when there is duplication of a name.
	 *
	 * @param in_ch Input channel object of InPort<T>
	 * @return true/false: "false" will be returned if registration goes wrong.
	 *
	 * @endif
	 */
	bool registerPort(InPortBase& in_ch)
	{
	  return registerInPort(in_ch);
	};
	
	/*!
	 * @if jp
	 *
	 * @brief InPort Ͽ
	 *
	 * ϿƤ InPort Ͽ롣
	 *
	 * @param in_ch InPort object of InPort<T>
	 *
	 * @else
	 *
	 * @brief Deletion of InPort
	 *
	 * "false" is returned when the object of the argument is not registered. 
	 *
	 * @param in_ch Input channel object of InPort<T>
	 * @return true/false: "false" will be returned if deletion goes wrong.
	 *
	 * @endif
	 */
	bool deleteInPort(InPortBase& in_ch);

	/*!
	 * @if jp
	 *
	 * @brief InPort Ͽ
	 *
	 * ϿƤ InPort Ͽ롣
	 *
	 * @param in_ch InPort object of InPort<T>
	 *
	 * @else
	 *
	 * @brief Deletion of InPort
	 *
	 * "false" is returned when the object of the argument is not registered. 
	 *
	 * @param in_ch Input channel object of InPort<T>
	 * @return true/false: "false" will be returned if deletion goes wrong.
	 *
	 * @endif
	 */
	bool deletePort(InPortBase& in_ch)
	{
	  return deleteInPort(in_ch);
	};
	
	/*!
	 * @if jp
	 *
	 * @brief InPort Ͽ
	 *
	 * ϿƤ InPort Ͽ̾ꤷƲ롣
	 *
	 * @param ch_name InPort ̾
	 *
	 * @else
	 *
	 * @brief Deletion of input channel by channel name.
	 *
	 * "false" is returned when the object of the argument is not registered. 
	 *
	 * @param ch_name Input channel name
	 * @return true/false: "false" will be returned if deletion goes wrong.
	 *
	 * @endif
	 */
	bool deleteInPortByName(const char* ch_name);

	/*!
	 * @if jp
	 *
	 * @brief Ƥ InPort Υǡ
	 *
	 * ƤInPortФ InPort::read() ¹ԡͽХɤ줿ѿ
	 * ǿͤ롣ѿ˥ХɤƤʤInPortǤϲ
	 * ʤ
	 *
	 * @else
	 *
	 * @brief Read all data to InPorts and bound variable
	 *
	 * Invoke InPort::read() to all registered InPorts.
	 *
	 * @endif
	 */	
	void readAllInPorts();


	/*!
	 * @if jp
	 *
	 * @brief Ƥ InPort νλ
	 *
	 * ƤInPortФƥ֥Ȥ deactivate() Ԥ
	 *
	 * @else
	 *
	 * @brief Finalize all InPorts
	 *
	 * Finalize all registered InPorts and deactivate as a CORBA objects.
	 *
	 * @endif
	 */	
	void finalizeInPorts();

	//============================================================
	// Output channel handling
	//============================================================
	/*!
	 * @if jp
	 *
	 * @brief OutPort Ͽ
	 *
	 * OutPort 򥳥ݡͥȤϿ롣Ͽ줿OutPortϳ
	 * 褦ˤʤ롣
	 *
	 * @param out_ch OutPort object of OutPort<T>
	 *
	 * @else
	 *
	 * @brief Registration of output channel
	 *
	 * Duplication of a name of output channel is not allowed.
	 * "false" will be returned when there is duplication of a name.
	 *
	 * @param out_ch Output channel object of OutPort<T>
	 * @return true/false: "false" will be returned if registration goes wrong.
	 *
	 * @endif
	 */
	bool registerOutPort(OutPortBase& out_ch);
	bool registerPort(OutPortBase& out_ch)
	{
	  return registerOutPort(out_ch);
	};
	
	/*!
	 * @if jp
	 *
	 * @brief OutPort Ͽ
	 *
	 * OutPort 򥳥ݡͥȤϿ롣Ͽ줿OutPortϳ
	 * 褦ˤʤ롣
	 *
	 * @param out_ch OutPort object of OutPort<T>
	 *
	 * @else
	 *
	 * @brief Deletion of output channel.
	 *
	 * "false" is returned when the object of the argument is not registered. 
	 *
	 * @param out_ch Output channel object of OutPort<T>
	 * @return true/false: "false" will be returned if deletion goes wrong.
	 *
	 * @endif
	 */
	bool deleteOutPort(OutPortBase& out_ch);

	/*!
	 * @if jp
	 *
	 * @brief OutPort Ͽ
	 *
	 * OutPort 򥳥ݡͥȤϿ롣Ͽ줿OutPortϳ
	 * 褦ˤʤ롣
	 *
	 * @param out_ch OutPort object of OutPort<T>
	 *
	 * @else
	 *
	 * @brief Deletion of output channel.
	 *
	 * "false" is returned when the object of the argument is not registered. 
	 *
	 * @param out_ch Output channel object of OutPort<T>
	 * @return true/false: "false" will be returned if deletion goes wrong.
	 *
	 * @endif
	 */
	bool deletePort(OutPortBase& out_ch)
	{
	  return deleteOutPort(out_ch);
	};
	
	/*!
	 * @if jp
	 *
	 * @brief OutPort Ͽ
	 *
	 * ϿƤ OutPort Ͽ̾ꤷƲ롣
	 *
	 * @param ch_name OutPort ̾
	 *
	 * @else
	 *
	 * @brief Deletion of output channel by channel name.
	 *
	 * "false" is returned when the object of the argument is not registered. 
	 *
	 * @param ch_name Output channel name
	 * @return true/false: "false" will be returned if deletion goes wrong.
	 *
	 * @endif
	 */
	bool deleteOutPortByName(const char* ch_name);

	/*!
	 * @if jp
	 *
	 * @brief Ƥ OutPort Υǡ񤭽Ф
	 *
	 * ƤOutPortФ OutPort::write() ¹ԡͽХɤ줿ѿ
	 * ͤ򥵥֥饤Ф˽񤭽ФΥХåե˽񤭹ޤ롣
	 * ѿ˥ХɤƤʤInPortǤϲⵯʤ
	 *
	 * @else
	 *
	 * @brief Write all data to OutPorts from bound variables
	 *
	 * Invoke OutPort::write() to all registered OutPorts.
	 *
	 * @endif
	 */	
	void writeAllOutPorts();

	
	/*!
	 * @if jp
	 *
	 * @brief Ƥ OutPort νλ
	 *
	 * ƤOutPortФƳƥ֥饤ФǤ֥Ȥ
	 * deactivate() Ԥ
	 *
	 * @else
	 *
	 * @brief Finalize all OutPorts
	 *
	 * Finalize all registered OutPorts and deactivate as a CORBA objects.
	 *
	 * @endif
	 */	
	void finalizeOutPorts();

	
	/*!
	 * @if jp
	 *
	 * @brief ݡͥ̾ alias Ͽ
	 *
	 * ݡͥ̾ alias Ͽ롣Ͽ줿 alias ϥ͡ߥ󥰥
	 * Ͽ롣
	 * ̾դ ƥȤζڤ "/" id  kind ζڤ "|"
	 * Ȥ롣ƥ Manipulator β˼ʬȤ MyManipulator0 Ȥ
	 * bind ˤϡ"/Manipualtor/MyManipulator0|rtc" ʸȤ
	 * Ϥޥ͡ˤŬڤʥߥ󥰤ǥ͡ߥ󥰥ФϿ롣
	 *
	 * @param alias ݡͥ̾ alias
	 *
	 * @else
	 *
	 * @brief Register alias of component name
	 *
	 * This method register alias of component name. Registered alias will
	 * bind to naming server.
	 * Context delimiter is "/", id/kind delimiter is "|".
	 * If you want to bind your component "MyManipulator0" under a context
	 * "Manipulator", you should give an char* argument 
	 * like "/Manipulator/MyManipulator|rtc".
	 * This component will be bound to naming server on this alias.
	 *
	 * @param alias An alias of this component name
	 *
	 * @endif
	 */
	void appendAlias(const char* alias);

	/*!
	 * @if jp
	 *
	 * @brief ݡͥ̾ alias Ͽ
	 *
	 * ݡͥ̾ alias Ͽ롣Ͽ줿 alias ϥ͡ߥ󥰥
	 * Ͽ롣
	 * ̾դ ƥȤζڤ "/" id  kind ζڤ "|"
	 * Ȥ롣ƥ Manipulator β˼ʬȤ MyManipulator0 Ȥ
	 * bind ˤϡ"/Manipualtor/MyManipulator0|rtc" ʸȤ
	 * Ϥޥ͡ˤŬڤʥߥ󥰤ǥ͡ߥ󥰥ФϿ롣
	 *
	 * @param alias ݡͥ̾ alias
	 *
	 * @else
	 *
	 * @brief Register alias of component name
	 *
	 * This method register alias of component name. Registered alias will
	 * bind to naming server.
	 * Context delimiter is "/", id/kind delimiter is "|".
	 * If you want to bind your component "MyManipulator0" under a context
	 * "Manipulator", you should give an char* argument 
	 * like "/Manipulator/MyManipulator|rtc".
	 * This component will be bound to naming server on this alias.
	 *
	 * @param alias An alias of this component name
	 *
	 * @endif
	 */
	void appendAlias(const std::string alias);

	/*!
	 * @if jp
	 *
	 * @brief ݡͥ̾ alias 
	 *
	 * ϿƤ륳ݡͥ̾ alias 롣
	 *
	 * @else
	 *
	 * @brief Get registered aliases of component name
	 *
	 * @endif
	 */
	std::list<string> getAliases();

	void setNamingPolicy(NamingPolicy policy);
	NamingPolicy getNamingPolicy();
	bool isLongNameEnable();
	bool isAliasEnable();

	void forceExit();

	/*!
	 * @if jp
	 *
	 * @brief ݡͥȤνλ
	 *
	 * 1.OutPortνλ+deactivate2.InPortνλ+deactivate
	 * 3.ƥӥƥߡ4.ݡͥȤdeactivate
	 * 5. 
	 *
	 * @else
	 *
	 * @brief Get registered aliases of component name
	 *
	 * @endif
	 */
	void finalize();
  
	bool isThreadRunning();
	
	

	
  protected:
	/*!
	 * @if jp
	 * @brief ORB ݥѿ
	 * @else
	 * @brief pointer to ORB
	 * @endif
	 */
	CORBA::ORB_ptr m_pORB;

	/*!
	 * @if jp
	 * @brief POA ݥѿ
	 * @else
	 * @brief pointer to POA
	 * @endif
	 */
	PortableServer::POA_ptr m_pPOA;

	/*!
	 * @if jp
	 * @brief Manager ݥѿ
	 * @else
	 * @brief pointer to Manager
	 * @endif
	 */
	RtcManager* m_pManager;

	/*!
	 * @if jp
	 * @brief ƥݡͥȤΥ֥ȥե
	 * @else
	 * @brief object reference to parent component
	 * @endif
	 */
	RTCBase_var m_Parent;

	/*!
	 * @if jp
	 * @brief ƥӥƥåɾ֥ե饰¤
	 * @else
	 * @brief activity thread state structure
	 * @endif
	 */
	typedef enum ThreadStates {
	  UNKNOWN,
	  RUNNING,
	  SUSPEND,
	  STOP
	};
	class ThreadState
	{
	public:
	  ThreadState() : m_Flag(RUNNING){};
	  ThreadStates m_Flag;
	  RTC_Thread_Mutex m_Mutex;
	};
	/*!
	 * @if jp
	 * @brief ƥӥƥåɾѿ
	 * @else
	 * @brief activity thread state variable
	 * @endif
	 */
	ThreadState m_ThreadState;

	/*!
	 * @if jp
	 * @brief ݡͥ̾ Fanctor 饹
	 * @else
	 * @brief Component name comparison fanctor class
	 * @endif
	 */
	class eq_name
	{
	public:
	  const string m_name;
	  eq_name(const char* name) : m_name(name){};
	  bool operator()(InPortBase* ch)
	  {
		string ch_name(ch->name());
		return m_name == ch_name;
	  };
	  bool operator()(OutPortBase* ch)
	  {
		string ch_name(ch->name());
		return m_name == ch_name;
	  };
	};

	/*!
	 * @if jp
	 * @brief ƥӥƥѿ饹
	 * @else
	 * @brief activity state varible structure
	 * @endif
	 */
	struct ComponentStateMtx {
	public:
	  /*!
	   * @if jp
	   * @brief ƥӥƥѿ饹󥹥ȥ饯
	   * @else
	   * @brief activity state varible structure constructor
	   * @endif
	   */
	  ComponentStateMtx():_state(RTM::RTComponent::RTC_BORN){};

	  /*!
	   * @if jp
	   * @brief ƥӥƥ
	   * @else
	   * @brief activity state varible structure
	   * @endif
	   */
	  ComponentState _state;
	  
	  /*!
	   * @if jp
	   * @brief ƥӥƥѿMutex
	   * @else
	   * @brief activity state varible mutex
	   * @endif
	   */
	  RTC_Thread_Mutex _mutex;
	};

	/*!
	 * @if jp
	 * @brief ƥӥƥθ߾ѿ
	 * @else
	 * @brief activity current state varible
	 * @endif
	 */
	ComponentStateMtx m_CurrentState;

	/*!
	 * @if jp
	 * @brief ƥӥƥμѿ
	 * @else
	 * @brief activity next state varible
	 * @endif
	 */
	ComponentStateMtx m_NextState;
	
	/*!
	 * @if jp
	 * @brief ƥӥƥؿݥ
	 * @else
	 * @brief activity function pointer definition
	 * @endif
	 */
	typedef RtmRes (RtcBase::*StateFunc)();
	
	//------------------------------------------------------------
	// Function pointer tables for state action functions.
	//------------------------------------------------------------
	/*!
	 * @if jp
	 * @brief ƥӥƥ exit ؿơ֥
	 * @else
	 * @brief exit function table of activity
	 * @endif
	 */
	StateFunc _exit_func[11];

	/*!
	 * @if jp
	 * @brief ƥӥƥ entry ؿơ֥
	 * @else
	 * @brief entry function table of activity
	 * @endif
	 */
	StateFunc _entry_func[11];

	/*!
	 * @if jp
	 * @brief ƥӥƥ do ؿơ֥
	 * @else
	 * @brief do function table of activity
	 * @endif
	 */
	StateFunc _do_func[11];
	
	//------------------------------------------------------------
	// State action functions
	//------------------------------------------------------------
	// 
	/*!
	 * @if jp
	 * @brief ƥӥƥ顼åؿ
	 * @else
	 * @brief activity error check function
	 * @endif
	 */
	RtmRes _check_error(RtmRes result);

	/*!
	 * @if jp
	 * @brief ƥӥƥѥߡؿ
	 * @else
	 * @brief dummy function for activity
	 * @endif
	 */
	RtmRes _nop(){return RTM_OK;}

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_init_entry ¹Դؿ
	 * @else
	 * @brief rtc_init_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_initializing();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_ready_entry ¹Դؿ
	 * @else
	 * @brief rtc_ready_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_ready_entry();
	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_starting_entry ¹Դؿ
	 * @else
	 * @brief rtc_starting_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_starting();
	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_active_entry ¹Դؿ
	 * @else
	 * @brief rtc_active_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_active_entry();
	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_stippoing_entry ¹Դؿ
	 * @else
	 * @brief rtc_stopping_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_stopping();
	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_aborting_entry ¹Դؿ
	 * @else
	 * @brief rtc_aborting_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_aborting();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_error_entry ¹Դؿ
	 * @else
	 * @brief rtc_error_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_error_entry();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_fatal_entry ¹Դؿ
	 * @else
	 * @brief rtc_fatal_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_fatal_entry();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_exiting_entry ¹Դؿ
	 * @else
	 * @brief rtc_exiting_entry execution function for activity
	 * @endif
	 */
	RtmRes _rtc_exiting();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_ready_do ¹Դؿ
	 * @else
	 * @brief rtc_ready_do execution function for activity
	 * @endif
	 */
	RtmRes _rtc_ready();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_active_do ¹Դؿ
	 * @else
	 * @brief rtc_active_do execution function for activity
	 * @endif
	 */
	RtmRes _rtc_active();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_error_do ¹Դؿ
	 * @else
	 * @brief rtc_error_do execution function for activity
	 * @endif
	 */
	RtmRes _rtc_error();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_fatal_do ¹Դؿ
	 * @else
	 * @brief rtc_fatal_do execution function for activity
	 * @endif
	 */
	RtmRes _rtc_fatal();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_ready_exit ¹Դؿ
	 * @else
	 * @brief rtc_ready_exit execution function for activity
	 * @endif
	 */
	RtmRes _rtc_ready_exit();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_active_exit ¹Դؿ
	 * @else
	 * @brief rtc_active_exit execution function for activity
	 * @endif
	 */
	RtmRes _rtc_active_exit();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_error_exit ¹Դؿ
	 * @else
	 * @brief rtc_error_exit execution function for activity
	 * @endif
	 */
	RtmRes _rtc_error_exit();

	/*!
	 * @if jp
	 * @brief ƥӥƥ rtc_fatal_exit ¹Դؿ
	 * @else
	 * @brief rtc_fatal_exit execution function for activity
	 * @endif
	 */
	RtmRes _rtc_fatal_exit();
	
	/*!
	 * @if jp
	 * @brief ƥӥƥؿơ֥ν
	 * @else
	 * @brief initialize activity function table
	 * @endif
	 */
	void init_state_func_table();
	

	/*!
	 * @if jp
	 * @brief InPort ꥹȥƥ졼
	 * @else
	 * @brief InPort list iterator
	 * @endif
	 */
	typedef list<InPortBase*>::iterator InPorts_it;
	
	/*!
	 * @if jp
	 * @brief OutPort ꥹȥƥ졼
	 * @else
	 * @brief OutPort list iterator
	 * @endif
	 */
	typedef list<OutPortBase*>::iterator OutPorts_it;
	
	/*!
	 * @if jp
	 * @brief mutex դ InPort ꥹȹ¤
	 * @else
	 * @brief InPort list structure with mutex 
	 * @endif
	 */
	struct InPorts
	{
	  list<InPortBase*> m_List;
	  RTC_Thread_Mutex m_Mutex;
	};
	/*!
	 * @if jp
	 * @brief mutex դ InPort ꥹ
	 * @else
	 * @brief InPort list with mutex 
	 * @endif
	 */
	InPorts m_InPorts;
	
	/*!
	 * @if jp
	 * @brief mutex դ OutPort ꥹȹ¤
	 * @else
	 * @brief OutPort list structure with mutex 
	 * @endif
	 */
	struct OutPorts
	{
	  list<OutPortBase*> m_List;
	  RTC_Thread_Mutex m_Mutex;
	};
	/*!
	 * @if jp
	 * @brief mutex դ OutPort ꥹ
	 * @else
	 * @brief OutPort list with mutex
	 * @endif
	 */
	OutPorts m_OutPorts;
	
	//! Input port flag list
	//  list<const bool*> m_InFlags;
	
	/*!
	 * @if jp
	 * @brief ƥӥƥѿ
	 * @else
	 * @brief activity state variable
	 * @endif
	 */
	TimedState m_TimedState;
	/*!
	 * @if jp
	 * @brief ƥӥƥOutPort
	 * @else
	 * @brief activity state OutPort
	 * @endif
	 */
	OutPortAny<TimedState> m_StatePort;									  

	/*!
	 * @if jp
	 * @brief ݡͥȥץե
	 * @else
	 * @brief component profile
	 * @endif
	 */
	RtcModuleProfile m_Profile;
	//	RtcManagerProfile m_ManagerProfile;
	//	RtcComponentProfile m_Profile;
	//	RtcConfiguration m_Configuration;

	/*!
	 * @if jp
	 * @brief ݡͥ̾alias
	 * @else
	 * @brief Aliases of component name
	 * @endif
	 */
	std::list<string> m_Alias;

	NamingPolicy m_NamingPolicy;

 	/*!
	 * @if jp
	 * @brief Хåե
	 * @else
	 * @brief Logger mediation buffer
	 * @endif
	 */
	RtcMedLogbuf m_MedLogbuf;

 	/*!
	 * @if jp
	 * @brief ȥ꡼
	 * @else
	 * @brief Logger stream
	 * @endif
	 */
	RtcLogStream rtcout;

  };

  //------------------------------------------------------------
  // RTComponent factory function type
  //------------------------------------------------------------

  class RtcManager;
	/*!
	 * @if jp
	 * @brief ݡͥȥ֥ȥեȥؿ
	 * @else
	 * @brief component object factory function declaration
	 * @endif
	 */
  typedef RtcBase* (*RtcNewFunc)(RtcManager* manager);

	/*!
	 * @if jp
	 * @brief ݡͥȥ֥Ѵؿ
	 * @else
	 * @brief component object delete function declaration
	 * @endif
	 */
  typedef void (*RtcDeleteFunc)(RtcBase* rtc);

};  // end of namespace RTM


#endif // __RtcBase_h__
