// -*- C++ -*-
/*!
 * @file RtcNameComponent.h
 * @if jp
 * @brief Name component إѡ饹
 * @else
 * @brief Name component helper class
 * @endif
 *
 * @date $Date$
 *
 * Copyright (c) 2003-2004 Noriaki Ando <n-ando@aist.go.jp>
 *          Task-intelligence Research Group,
 *          Intelligent System Research Institute,
 *          National Institute of Industrial Science (AIST), Japan
 *          All rights reserved.
 *
 * $Id$
 */


#ifndef __RtcNameComponent_h__
#define __RtcNameComponent_h__

// CORBA header include
#include "rtm/idl/RTMBaseStub.h"
#include "rtm/idl/RTMBaseSkel.h"

// STL includes
#include <string>
#include <vector>

namespace RTM
{

  /*!
   * @if jp
   *
   * @class RtcNameComponent
   *
   * @brief CORBA Naming Service Ѥ Name Component إѡ饹
   *
   * @else
   *
   * @class RtcNameComponent
   *
   * @brief Name component handling class for CORBA Naming Service.
   *
   * @endif
   */
  class RtcNameComponent
  {
  public:
	/*!
	 * @if jp
	 * @brief RtcNaming 饹󥹥ȥ饯
	 *
	 * @else
	 *
	 * @brief A constructor of RtcNaming class.
	 *
	 * @endif
	 */
	RtcNameComponent();

	/*!
	 * @if jp
	 * @brief Name Component 
	 *
	 * Ϳ줿ʸ󤫤 Name Component 롣
	 * 
     * @param path Name Component ѥʸ
	 *
	 * @else
	 *
	 * @brief Create name component
	 *
     * Create a name component from given string.
	 *
     * @param path path string to be name component.
	 *
	 * @endif
	 */
	RtcNameComponent(const std::string path);
	
	/*!
	 * @if jp
	 * @brief Name Component 
	 *
	 * Ϳ줿ʸ󤫤 Name Component 롣
	 * 
     * @param path Name Component ѥʸ
	 *
	 * @else
	 *
	 * @brief Create name component
	 *
     * Create a naming component from given string.
	 *
     * @param path path string to be name component.
	 *
	 * @endif
	 */
	RtcNameComponent(const char* path);

	/*!
	 * @if jp
	 * @brief Name Component μ
	 *
	 * ݻƤ Name Component 롣
	 * 
	 * @else
	 *
	 * @brief Get current name component
	 *
     * Create a name component from given string.
	 *
     * @param path path string to be name component.
	 *
	 * @endif
	 */
	CosNaming::Name get();

	/*!
	 * @if jp
	 * @brief Name Component pop
	 *
	 * ݻƤ Name Component κpop롣
	 * 
	 * @else
	 *
	 * @brief Pop the last name component.
	 *
     * Pop and return the last name component from current name components.
	 *
	 * @endif
	 */
	CosNaming::Name pop();

	/*!
	 * @if jp
	 * @brief Name Component  push
	 *
	 * ݻƤ Name Component κ push 롣
	 * 
	 * @else
	 *
	 * @brief Push the given string.
	 *
     * Push the given string to the last of name components.
	 *
	 * @endif
	 */
	CosNaming::Name push(const std::string& path);

	/*!
	 * @if jp
	 * @brief Name Component ξõ
	 *
	 * ݻƤ Name Component õ롣
	 * 
	 * @else
	 *
	 * @brief Erase the current name components.
	 *
     * Erase the curent name component which this object is holding.
	 *
	 * @endif
	 */
	void erase();

	inline int length() {return m_Length;}

	/*!
	 * @if jp
	 * @brief Name Component κ
	 *
	 * Ϳ줿ʸ󤫤 Name Component ŪХ᥽åɡ
	 * 
	 * @else
	 *
	 * @brief Create name component.
	 *
     * Static member method which creates name component from given path string.
	 *
	 * @endif
	 */
	static CosNaming::Name stringToNameComponent(const std::string& path);

  private:
	/*!
	 * @if jp
	 * @brief ʸʬ
	 *
	 * Ϳ줿ʸ󤫤 Name Component ŪХ᥽åɡ
	 * 
     * @param input ʸ std::string 
     * @param delimiter ǥߥ std::string 
     * @param results ̡ std::vector<std::string> 
	 *
	 * @else
	 *
	 * @brief Split string.
	 *
     * Static member method which creates name component from given path string.
	 *
     * @param input Input string to be split. std::string tyoe.
     * @param delimiter Delimter. std::string type.
     * @param results Results. std::vector<std::string> type.
	 *
	 * @endif
	 */
	static int split(const std::string& input, const std::string& delimiter,
					 std::vector<std::string>& results);

	static void nodeToIdKind(const std::string& node, std::string& id,
							 std::string& kind);

	/*!
	 * @if jp
	 * @brief Name Component
	 *
	 * ֥ݻ name component
	 * 
	 * @else
	 *
	 * @brief name Component
	 *
     * A name component to be hold by this object.
	 *
	 * @endif
	 */
	CosNaming::Name m_Name;
	/*!
	 * @if jp
	 * @brief Name Component Ĺ
	 *
	 * ֥ݻ name component Ĺ
	 * 
	 * @else
	 *
	 * @brief Length of the name component
	 *
     * The length of the name component.
	 *
	 * @endif
	 */
	int m_Length;
  };
};

#endif // __RtcNameComponent_h__
