// -*- C++ -*-
/*!
 * @file RtcOutPort.h
 * @if jp
 * @brief OutPort ƥץ졼ȥ饹
 * @else
 * @brief OutPort template class
 * @endif
 *
 * @date $Date$
 *
 * Copyright (c) 2003 Noriaki Ando <n-ando@aist.go.jp>
 *          Task-intelligence Research Group,
 *          Intelligent System Research Institute,
 *          National Institute of Industrial Science (AIST), Japan
 *          All rights reserved.
 *
 * $Id$
 */

#ifndef __RtcOutPort_h__
#define __RtcOutPort_h__

//#define ACE_HAS_CLOCK_GETTIME

#include "rtm/RtcOutPortBase.h"
#include "rtm/idl/RTCDataTypeSkel.h"
#include <iostream>

#define DEFAULT_BUFFER_SIZE 64

namespace RTM {
  /*!
   * @if jp
   *
   * @class OutPortAny
   *
   * @brief OutPort ƥץ졼ȥ饹
   * 
   * OutPort μǤ OutPortAny<T> Υƥץ졼ȥ饹
   * <T> RTCDataType.idl ˤƤ뷿ǡФȤ
   * Time  tm ,  T data Ĺ¤ΤǤʤƤϤʤʤ
   *
   * @else
   *
   * @class InPortAny
   *
   * @brief OutPort template class
   *
   * This class: OutPortAny<T> which is a implementations of OutPort
   * This class template provides interfaces to output channel.
   * Component developer can define output value, which act as output
   * channel to other components, using this template.
   * This is class template. This class have to be incarnated class as channel
   * value types. This value types are previously define RtComponent IDL.
   * ex. type T: TimedFload, TimedLong etc... 
   *
   * @endif
   */
  template <class T> class OutPortAny
	: public OutPortBase
  {
  public:
	/*!
	 * @if jp
	 *
	 * @brief OutPortAny 饹󥹥ȥ饯
	 *
	 * OutPortAny<T> 饹Υ󥹥ȥ饯
	 * ѥ᡼ȤͿ T ѿ˥Хɤ롣
	 *
	 * @param name OutPort ̾OutPortBase:name() ˤ껲Ȥ롣
	 * @param value  OutPort ˥Хɤ T ѿ
	 * @param bufsize OutPort Υ󥰥ХåեΥХåեĹ
	 *
	 * @else
	 *
	 * @brief A constructor.
	 *
	 * Setting channel name and registering channel value.
	 *
	 * @param name A name of the OutPort. This name is referred by
	 *             OutPortBase::name().
	 * @param value A channel value related with the channel.
	 * @param bufsize Buffer length of internal ring buffer of OutPort 
	 *
	 * @endif
	 */
	OutPortAny(const char* name, T& value, int bufsize=DEFAULT_BUFFER_SIZE)
	  : m_Buffer(bufsize, value), m_Value(value)
	{
	  // Set PortProfile
	  m_Profile.name = CORBA::string_dup(name);
	  CORBA::Any var;
	  var <<= m_Value;
	  m_Profile.port_type = var.type();
	  
	  // Initializing ring buffer
	  for (int i = 0 ; i < m_Buffer.buff_length(); i++)
		{
		  m_Buffer.put(m_Value);
		}
	}
	
	/*!
	 * @if jp
	 *
	 * @brief OutPortAny 饹󥹥ȥ饯
	 *
	 * OutPortAny<T> 饹Υ󥹥ȥ饯
	 * 
	 *
	 * @param name OutPort ̾OutPortBase:name() ˤ껲Ȥ롣
	 * @param bufsize OutPort Υ󥰥ХåեΥХåեĹ
	 *
	 * @else
	 *
	 * @brief A constructor.
	 *
	 * Setting channel name and registering channel value.
	 *
	 * @param name A name of the OutPort. This name is referred by
	 *             OutPortBase::name().
	 * @param bufsize Buffer length of internal ring buffer of OutPort 
	 *
	 * @endif
	 */
	OutPortAny(const char* name, int bufsize=DEFAULT_BUFFER_SIZE)
	  : m_Buffer(bufsize, m_Dummy), m_Value(m_Dummy)
	{
	  // Set PortProfile
	  m_Profile.name = CORBA::string_dup(name);
	  CORBA::Any var;
	  var <<= m_Value;
	  m_Profile.port_type = var.type();
	  
	  // Initializing ring buffer
	  for (int i = 0 ; i < m_Buffer.buff_length(); i++)
		{
		  m_Buffer.put(m_Value);
		}
	}
	
	
	/*!
	 * @if jp
	 *
	 * @brief OutPortAny 饹ǥȥ饯
	 *
	 * OutPortAny<T> 饹Υǥȥ饯
	 *
	 * @else
	 *
	 * @brief A destructor
	 *
	 * @endif
	 */
	virtual ~OutPortAny(){};
	
	/*!
	 * @if jp
	 *
	 * @brief [CORBA interface] ߤ OutPort κǿͤ
	 *
	 * ߤ OutPort κǿͤ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get current data of OutPort
	 *
	 * @endif
	 */
	CORBA::Any* get()
	  throw (CORBA::SystemException)
	{
//	  ACE_TRACE("OutPortBase::get()");
	  CORBA::Any_var tmp = new CORBA::Any();
	  tmp <<= m_Buffer.get_new();
	  return tmp._retn();
	}
	
	/*!
	 * @if jp
	 *
	 * @brief ߤ OutPort ͤ򥵥֥饤Фž
	 *
	 * ߤ OutPort ͤ򥵥֥饤Фž
	 *
	 * @else
	 *
	 * @brief Push current data to subscribers
	 *
	 * @endif
	 */
	//	RtmRes push(const InPort_ptr& inport, SubscriptionID subsid)
	RtmRes push(const InPort_ptr& inport, std::string& subsid)
	{
//	  ACE_TRACE("OutPortBase::push()");
	  
	  if (CORBA::is_nil(inport)) 
		{
//		  ACE_DEBUG((LM_DEBUG, "Invalid object reference: inport"));
		  return RTM_ERR;
		}

	  try
		{
		  CORBA::Any val;
		  val <<= m_Buffer.get_new();
		  
		  if (inport != NULL)
			{
			  inport->put(val);
			}
		}
	  catch(CORBA::SystemException& e)
		{
		  //		  cout << "push(): CORBA System Exception." << endl;

		  // Subscribers already locked
		  unsubscribeNoLocked(subsid.c_str());
		  return RTM_ERR;
		}
	  catch(RTM::InPort::Disconnected& e)
		{
		  //		  ACE_DEBUG((LM_DEBUG, "Disconnected."));

		  // Subscribers already locked
		  unsubscribeNoLocked(subsid.c_str());
		  return RTM_ERR;
		}
	  return RTM_OK;
	}
	
	
	
	/*!
	 * @if jp
	 *
	 * @brief OutPort Υ󥰥Хåեͤ
	 *
	 * OutPort Υ󥰥Хåեͤ롣
	 *
	 * @else
	 *
	 * @brief Initialize ring buffer value of OutPort
	 *
	 * @endif
	 */
	virtual void initBuffer(T& value)
	{
	  for (int i = 0 ; i <= m_Buffer.buff_length(); i++)
		{
		  m_Buffer.put(value);
		}
	}
	
	
	/*!
	 * @if jp
	 *
	 * @brief Хɤ줿 T Υǡ OutPort κǿͤȤƽ񤭹
	 *
	 * Хɤ줿 T Υǡ OutPort κǿͤȤƽ񤭹ࡣ
	 * 󥹥ȥ饯 T ѿ OutPort ХɤƤɬפ롣
	 *
	 * @else
	 *
	 * @brief Write into current OutPort from bound T-type data
	 *
	 * @endif
	 */
	virtual void write()
	{
	  struct timespec ts;
//	  ACE_OS::clock_gettime(CLOCK_REALTIME, &ts);
	  clock_gettime(CLOCK_REALTIME, &ts);
	  m_Value.tm.sec = ts.tv_sec;
	  m_Value.tm.nsec = ts.tv_nsec;
	  m_Buffer.put(m_Value);
	  updateall();
	};


	/*!
	 * @if jp
	 *
	 * @brief Хɤ줿 T Υǡ OutPort κǿͤȤƽ񤭹
	 *
	 * Хɤ줿 T Υǡ OutPort κǿͤȤƽ񤭹ࡣ
	 * 󥹥ȥ饯 T ѿ OutPort ХɤƤɬפ롣
	 * Υ᥽åɤϥݥ⡼եå˻ѤȤƤ뤿ᡢ
	 * ˰¸ʤͤȤʤäƤ롣
	 *
	 * @else
	 *
	 * @brief Write into current OutPort from bound T-type data
	 *
	 * @endif
	 */
	virtual void write_pm()
	{
	  write();
	};
	
	
	/*!
	 * @if jp
	 *
	 * @brief T Υǡ OutPort κǿͤȤƽ񤭹
	 *
	 * ȤͿ줿 T Υǡ OutPort κǿͤȤƽ񤭹ࡣ
	 *
	 * @param value OutPort Хåե˽񤭹 T 
	 *
	 * @else
	 *
	 * @brief Writing a argument value to OutPort buffer
	 *
	 * Calling this method, argument value "T& value" is transferred to OutPort.
	 *
	 * @param value type-T value write into OutPort buffer
	 *
	 * @endif
	 */
	virtual void write(T value)
	{
	  struct timespec ts;
	  //ACE_OS::clock_gettime(CLOCK_REALTIME, &ts);
	  clock_gettime(CLOCK_REALTIME, &ts);
	  value.tm.sec = ts.tv_sec;
	  value.tm.nsec = ts.tv_nsec;
	  m_Buffer.put(value);
	  updateall();
	};
	
	
	/*!
	 * @if jp
	 *
	 * @brief T Υǡ OutPort κǿͤȤƽ񤭹
	 *
	 * ȤͿ줿 T Υǡ OutPort κǿͤȤƽ񤭹ࡣ
	 *
	 * @param value OutPort Хåե˽񤭹 T 
	 *
	 * @else
	 *
	 * @brief Writing a argument value to OutPort buffer
	 *
	 * Calling this method, argument value "T& value" is transferred to OutPort.
	 *
	 * @param value type-T value write into OutPort buffer
	 *
	 * @endif
	 */
	virtual void operator<<(T& value)
	{
	  struct timespec ts;
	  //ACE_OS::clock_gettime(CLOCK_REALTIME, &ts);
	  clock_gettime(CLOCK_REALTIME, &ts);
	  value.tm.sec = ts.tv_sec;
	  value.tm.nsec = ts.tv_nsec;
	  m_Buffer.put(value);
	  updateall();
	}
	
	
  private:
	/*!
	 * @if jp
	 * @brief Хɤ T ѿؤλ
	 * @else
	 * @brief reference to type-T value bound this OutPort
	 * @endif
	 */
	T& m_Value;
	
	/*!
	 * @if jp
	 * @brief T Υߡѿ
	 * @else
	 * @brief Dummy variable of type-T
	 * @endif
	 */
	T m_Dummy;
	
	/*!
	 * @if jp
	 * @brief Хåե
	 * @else
	 * @brief Buffer
	 * @endif
	 */
	RingBuffer<T> m_Buffer;
  };
  
}; // End of namesepace RTM

#endif // __RtcOutputChannel_h__
