// -*- C++ -*-
/*!
 * @file RtcOutPortBase.h
 * @if jp
 * @brief InPortBase 쥯饹
 * @else
 * @brief InPortBase base class
 * @endif
 *
 * @date $Date$
 *
 * Copyright (c) 2003,2004 Noriaki Ando <n-ando@aist.go.jp>
 *          Task-intelligence Research Group,
 *          Intelligent System Research Institute,
 *          National Institute of Industrial Science (AIST), Japan
 *          All rights reserved.
 *
 * $Id$
 */

#ifndef __RtcOutPortBase_h__
#define __RtcOutPortBase_h__

#include <list>

#include "rtm/RTC.h"
#include "rtm/idl/RTCDataTypeSkel.h"
#include "rtm/idl/RTCOutPortSkel.h"
#include "rtm/idl/RTCOutPortStub.h"
#include "rtm/RtcInPortBase.h"
#include "rtm/RtcRingBuffer.h"

#include "rtm/Rtc_Mutex.h"
#include "rtm/Rtc_Task.h"

namespace RTM {
  class SubscriberBase;
  using namespace std;

  /*!
   * @if jp
   *
   * @class OutPortBase
   *
   * @brief OutPort 쥯饹
   * 
   * OutPort μǤ OutPort<T> δ쥯饹
   * CORBA interface ؤμ󶡤롣
   *
   * @else
   *
   * @class OutPortBase
   *
   * @brief Output port base class.
   *
   * The base class of OutPort<T> s which are implementations of OutPort  
   * This class provides CORBA interface implementation.
   *
   * @endif
   */
  class OutPortBase :
	public POA_RTM::OutPort,
	public PortableServer::RefCountServantBase
  {
  public:
	/*!
	 * @if jp
	 * @brief OutPortBase 饹󥹥ȥ饯
	 *
	 * OutPortBase Υ饹󥹥ȥ饯
	 *
	 * @else
	 *
	 * @brief A constructor of OutPortBase class.
	 *
	 * Constructor of OutPortBase.
	 *
	 * @endif
	 */
	OutPortBase(){};

	/*!
	 * @if jp
	 * @brief OutPortBase 饹ǥȥ饯
	 *
	 * OutPortBase Υ饹ǥȥ饯
	 *
	 * @else
	 *
	 * @brief A destructor of OutPortBase class.
	 *
	 * Destructor of OutPortBase.
	 *
	 * @endif
	 */
	virtual ~OutPortBase(){};

	//============================================================
	// CORBA interfaces.
	//============================================================
	/*!
	 * @if jp
	 * 
	 * @brief [CORBA interface] OutPort θͤ
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get OutPort current value
	 * 
	 * @endif
	 */
	virtual CORBA::Any* get() throw (CORBA::SystemException) = 0;

	//! CORBA interface Return type code of port value.
	//	virtual CORBA::TypeCode_ptr port_type();

	/*!
	 * @if jp
	 * 
	 * @brief OutPort 򥵥֥饤֤
	 *
	 * @else
	 *
	 * @brief Subscribe this OutPort
	 * 
	 * @endif
	 */
	virtual RtmRes subscribe(InPort_ptr in_port, SubscriptionID_out id,
							 const SubscriberProfile &profile)
	  throw (CORBA::SystemException);

	/*!
	 * @if jp
	 * 
	 * @brief OutPort Υ֥饤֤
	 *
	 * @else
	 *
	 * @brief Unsubscribe this OutPort
	 * 
	 * @endif
	 */
	virtual RtmRes unsubscribe(const char* id) throw (CORBA::SystemException);

	/*!
	 * @if jp
	 * 
	 * @brief ߥ֥饤֤Ƥ InPort ΥꥹȤ
	 *
	 * @else
	 *
	 * @brief Get InPort list of current subscriber
	 * 
	 * @endif
	 */
	virtual InPortList* inports() throw (CORBA::SystemException);

	/*!
	 * @if jp
	 * 
	 * @brief [CORBA interface] OutPort Υץե
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Get OutPort profile
	 * 
	 * @endif
	 */
	virtual PortProfile* profile() throw (CORBA::SystemException);

	// End of CORBA interfaces
	//============================================================

	/*!
	 * @if jp
	 * 
	 * @brief ߤ OutPort ͤ򥵥֥饤Фpush
	 *
	 * @else
	 *
	 * @brief Push current OutPort value to subscriber
	 * 
	 * @endif
	 */
	//	virtual RtmRes push(const InPort_ptr& inport, SubscriptionID subsid) = 0;
	virtual RtmRes push(const InPort_ptr& inport, std::string& subsid) = 0;

	/*!
	 * @if jp
	 * 
	 * @brief ߤ OutPort ͤ򥵥֥饤ФФƹ
	 *
	 * @else
	 *
	 * @brief Update current OutPort to to subscriber
	 * 
	 * @endif
	 */
	virtual void updateall();

	/*!
	 * @if jp
	 * 
	 * @brief ߤΥ֥饤Ф
	 *
	 * @else
	 *
	 * @brief Disconnect all subscribers
	 * 
	 * @endif
	 */
	virtual void disconnect_all();

	/*!
	 * @if jp
	 * 
	 * @brief OutPort ̾
	 *
	 * @else
	 *
	 * @brief Get OutPort name
	 * 
	 * @endif
	 */
	virtual const char* name() {return m_Profile.name;};

	/*!
	 * @if jp
	 *
	 * @brief Хɤ줿 T Υǡ OutPort κǿͤȤƽ񤭹
	 *
	 * 貾۴ؿ饹ˤꥪС饤ɤݥ⡼եå
	 * Ѥޤ᥽åɡ
	 *
	 * @else
	 *
	 * @brief Write into current OutPort from bound T-type data
	 *
	 * Pure virtual method.
	 *
	 * @endif
	 */
	virtual void write_pm() = 0;


  protected:
	virtual RtmRes unsubscribeNoLocked(const char* id);

	/*!
	 * @if jp
	 * @brief ֥饤ФΥꥹ
	 * @else
	 * @brief Subscriber list
	 * @endif
	 */
	//	list<SubscriberBase*> m_Subscribers;
	struct Subscribers
	{
	  RTC_Thread_Mutex m_Mutex;
	  list<SubscriberBase*> m_List;
	};
	Subscribers m_Subscribers;
	  

	/*!
	 * @if jp
	 * @brief OutPort Υץե
	 * @else
	 * @brief OutPort profile
	 * @endif
	 */
	PortProfile m_Profile;

  };


  
}; // End of namespace RTM

#endif // End of ifndef __RtcOutPortBase_h__
