/*! -*- C++ -*-
 *
 * @file RtcRingBuffer.h
 * @if jp
 * @brief Ring buffer ƥץ졼ȥ饹
 * @else
 * @brief Ring buffer template class
 * @endif
 *
 * Copyright (c) 2003,2004 Noriaki Ando <n-ando@aist.go.jp>
 *          Task-intelligence Research Group,
 *          Intelligent System Research Institute,
 *          National Institute of Industrial Science (AIST), Japan
 *          All rights reserved.
 *
 * $Id$
 */

#ifndef __RtcRingBuffer_h__
#define __RtcRingBuffer_h__

#include <vector>
#include <iostream>

#include "rtm/Rtc_Mutex.h"
#include "rtm/Rtc_Task.h"

namespace RTM {

  /*!
   * @if jp
   *
   * @class RingBuffer
   *
   * @brief RingBuffer ƥץ졼ȥ饹
   * 
   * ǤդηФƥ󥰥Хåե󶡤 RingBuffer ƥץ졼ȥ饹
   * ǤդĹΥХåեĹĤȤ롣
   * ǿ͡ǸͤμФǤդξѿμФԤ롣
   *
   * @else
   *
   * @class RingBuffer
   *
   * @brief RingBuffer template class
   *
   * RingBuffer template class which provides for any types variable.
   * This ring buffer is able to have any length of buffer.
   * You can get newest and oldest value, value in any position of buffer.
   *
   * @endif
   */

  template <class T> class RingBuffer
  {
  public:
	/*!
	 * @if jp
	 *
	 * @brief RingBuffer 饹󥹥ȥ饯
	 *
	 * RingBuffer<T> 饹Υ󥹥ȥ饯
	 * ѥ᡼ȤͿ T ѿΥХåե롣
	 *
	 * @param length ХåեĹ
	 *
	 * @else
	 *
	 * @brief RingBuffer constructor.
	 *
	 * RingBuffer<T> class constructor.
	 * This constructor create the buffer with "length".
	 *
	 * @param length Buffer length
	 *
	 * @endif
	 */
	RingBuffer(int length)
	  : m_Oldest(0), m_Newest(length)
	{
	  if (length > 1)
		{
		  m_Length = length;
		}
	  else
		{
		  m_Length = 2;
		}
	  m_Buffer.resize(m_Length);
	}
	
	/*!
	 * @if jp
	 *
	 * @brief RingBuffer 饹󥹥ȥ饯
	 *
	 * RingBuffer<T> 饹Υ󥹥ȥ饯
	 * ѥ᡼ȤͿ T ѿΥХåե롣
	 *
	 * @param length ХåեĹ
	 * @param inival 
	 *
	 * @else
	 *
	 * @brief RingBuffer constructor.
	 *
	 * RingBuffer<T> class constructor.
	 * This constructor create the buffer with "length".
	 *
	 * @param length Buffer length
	 * @param inival Initial value
	 *
	 * @endif
	 */
	RingBuffer(int length, T inival)
	  : m_Oldest(0), m_Newest(length)
	{
	  if (length > 1)
		{
		  m_Length = length;
		}
	  else
		{
		  m_Length = 2;
		}
	  
	  m_Buffer.resize(m_Length);
	  
	  for (int i = 0; i < m_Length; i++)
		{
		  m_Buffer[i] = inival;
		}
	}
	
	/*!
	 * @if jp
	 *
	 * @brief RingBuffer ͤǼ
	 *
	 * RingBuffer ˰ͤǼ롣
	 *
	 * @param value Ǽ T 
	 *
	 * @else
	 *
	 * @brief Store given value into the RingBuffer
	 *
	 * Store given argument value into the RingBuffer.
	 *
	 * @param value Stored value
	 *
	 * @endif
	 */
	inline void put(const T& value)
	{
	  m_Buffer[m_Oldest].write(value);
	  
	  RTC_Guard<RTC_Thread_Mutex> guard(m_Mutex);
	  m_Newest = m_Oldest;
	  m_Oldest = (++m_Oldest) % m_Length;

	}

	/*!
	 * @if jp
	 *
	 * @brief ǿͤ
	 *
	 * RingBuffer κǿͤ롣
	 *
	 * @else
	 *
	 * @brief Get the newset value
	 *
	 * Get the newest value stored in the RingBuffer.
	 *
	 * @endif
	 */
	inline const T& get_new()
	{
	  RTC_Guard<RTC_Thread_Mutex> guard(m_Mutex);

	  return m_Buffer[m_Newest].read();
	}


	/*!
	 * @if jp
	 *
	 * @brief ̤ɤοǡս(->)Ǽ
	 *
	 * @else
	 *
	 * @brief Get new data to be read.
	 *
	 * @endif
	 */
	inline std::vector<T> get_new_rlist()
	{
	  RTC_Guard<RTC_Thread_Mutex> guard(m_Mutex);

	  std::vector<T> data;
	  int cnt(0);
	  for (int i(m_Newest);
		   m_Buffer[i].is_new() && cnt < m_Length;
		   ++cnt, --i < 0 ? (i = m_Length - 1) : 0)
		{
		  data.push_back(m_Buffer[i].read());
		}

	  return data;
	}


	/*!
	 * @if jp
	 *
	 * @brief ̤ɤοǡ
	 *
	 * @else
	 *
	 * @brief Get new data to be read.
	 *
	 * @endif
	 */
	inline std::vector<T> get_new_list()
	{
	  RTC_Guard<RTC_Thread_Mutex> guard(m_Mutex);
	  
	  std::vector<T> data;
	  int len(new_data_len());
	  data.resize(len);
	  int npos;
	  npos = m_Newest - (len - 1) < 0 ?
		m_Newest - (len - 1) + m_Length  : m_Newest - (len - 1);
	  int cnt(0);
	  for ( ; cnt < len; ++cnt, ++npos > m_Length - 1 ? npos = 0 : 0)
		{
		  data[cnt] = m_Buffer[npos].read();
		}

	  return data;
	}

	/*!
	 * @if jp
	 *
	 * @brief ̤ɤοǡ
	 *
	 * @else
	 *
	 * @brief Get number of new data to be read.
	 *
	 * @endif
	 */
	inline int new_data_len()
	{
	  RTC_Guard<RTC_Thread_Mutex> guard(m_Mutex);

	  int cnt(0);
	  for (int i(m_Newest);
		   m_Buffer[i].is_new() && cnt < m_Length;
		   ++cnt, --i < 0 ? (i = m_Length - 1) : 0)
		{;}

	  return cnt;
	}


	/*!
	 * @if jp
	 *
	 * @brief ǸŤͤ
	 *
	 * RingBuffer κǸŤͤ롣
	 *
	 * @else
	 *
	 * @brief Get the oldest value
	 *
	 * Get the oldest value stored in the RingBuffer.
	 *
	 * @endif
	 */
	inline T& get_old()
	{
	  RTC_Guard<RTC_Thread_Mutex> guard(m_Mutex);

	  return m_Buffer[m_Oldest].read();
	}

	/*!
	 * @if jp
	 *
	 * @brief ǿͤ pos ĸŤͤ
	 *
	 * RingBuffer κǿͤ pos ĸŤͤ롣
	 *
	 * @else
	 *
	 * @brief Get pos-th older value from the newest value
	 *
	 * Get pos-th older value from the newset value stored in the RingBuffer.
	 *
	 * @endif
	 */
	virtual T& get_back(int pos)
	{
	  RTC_Guard<RTC_Thread_Mutex> guard(m_Mutex);
	  pos = pos % m_Length;

	  return m_Buffer[(m_Length + m_Newest - pos) % m_Length].read();
	}

	/*!
	 * @if jp
	 *
	 * @brief ǸŤͤ pos Ŀͤ
	 *
	 * RingBuffer κǸŤͤ pos Ŀͤ롣
	 *
	 * @else
	 *
	 * @brief Get pos-th newer value from the oldest value
	 *
	 * Get pos-th newer value from the oldest value stored in the RingBuffer.
	 *
	 * @endif
	 */
	inline T& get_front(int pos)
	{
	  RTC_Guard<RTC_Thread_Mutex> guard(m_Mutex);
	  pos = pos % m_Length;
	  
	  return m_Buffer[(m_Oldest + pos) % m_Length].read();
	}

	/*!
	 * @if jp
	 *
	 * @brief ХåեĹ
	 *
	 * ߤΥХåեĶ
	 *
	 * @else
	 *
	 * @brief Get buffer length
	 *
	 * Get buffer length
	 *
	 * @endif
	 */
	inline int buff_length()
	{
	  return m_Length;
	}

	/*!
	 * @if jp
	 *
	 * @brief ǿǡ̤ɤοǡɤĴ٤
	 *
	 * @else
	 *
	 * @brief Check the newest data is new data to be read.
	 *
	 * @endif
	 */
	inline bool is_new()
	{
	  return m_Buffer[m_Newest].is_new();
	}


  protected:
	/*!
	 * @if jp
	 * @brief Mutex
	 * @else
	 * @brief Mutex
	 * @endif
	 */
	RTC_Thread_Mutex m_Mutex;

	/*!
	 * @if jp
	 * @brief ХåեĹ
	 * @else
	 * @brief Buffer length
	 * @endif
	 */
	int m_Length;

	/*!
	 * @if jp
	 * @brief Хåե
	 * @else
	 * @brief Buffer sequence
	 * @endif
	 */
	template <class D> class Data
	{
	public:
	  Data() : _data(), _is_new(false){;}
	  inline Data& operator=(const D& other)
	  {
		this->_data = other;
		this->_is_new = true;
		return *this;
	  }
	  inline void write(const D& other)
	  {
		this->_is_new = true;
		this->_data = other;
	  }
	  inline D& read()
	  {
		this->_is_new = false;
		return this->_data;
	  }
	  inline bool is_new()
	  {
		return _is_new;
	  }
	protected:
	  D _data;
	  bool _is_new;
	};
	std::vector<Data<T> > m_Buffer;

	/*!
	 * @if jp
	 * @brief ǿͤΥХåե
	 * @else
	 * @brief Position of newset value stored in
	 * @endif
	 */
	int m_Newest;

	/*!
	 * @if jp
	 * @brief ǸͤΥХåե
	 * @else
	 * @brief Position of oldest value stored in
	 * @endif
	 */
	int m_Oldest;
  };
 
};


#endif // end of __RtcRingBuffer_h__
