/*
 *
 * Utilities for OpenRTM
 * Copyright(C) Isao Hara
 */

#include <iostream>
#include "Rtc_Util.h"


RTC_smatch::RTC_smatch(int n) : _max(n), _result(0) {
     this->_str = new std::string[n];
}

RTC_smatch::~RTC_smatch(){
      this->_max = 0;
      delete [] this->_str;
}

const char *RTC_smatch::str(int n){
      if(this->_result > n) return this->_str[n].c_str();
      else return NULL;
}


RTC_regex::RTC_regex(std::string pattern, int flag){
    int err;
     char errbuf[1024];
     err = regcomp(&this->_pattern, pattern.c_str(), flag);
     if(err){
       regerror(err, &this->_pattern, errbuf, 1024);
       std::cerr << ">Error " << err << " " <<errbuf  <<std::endl;
      }

}
RTC_regex::RTC_regex(const char *pattern, int flag){
    int err;
     char errbuf[1024];
     err = regcomp(&this->_pattern, pattern, flag);
     if(err){
       regerror(err, &this->_pattern, errbuf, 1024);
       std::cerr << ">Error " << err << " " <<errbuf  <<std::endl;
      }
}

RTC_regex::~RTC_regex(){
      regfree(&this->_pattern);
}

bool regex_match(std::string& cline, RTC_smatch& results, RTC_regex& preg){
  int result,i;
  regmatch_t pmatch[results._max];

  result = regexec(&preg._pattern, cline.c_str(), results._max, pmatch, 0);
  if(result == 0){
    for(i=0;;i++){
      if(pmatch[i].rm_so < 0) break ;
      results._str[i] = cline.substr(pmatch[i].rm_so, pmatch[i].rm_eo);
      results._result += 1;
    }
    return true;
  }else{
    return false;
  }
}

bool regex_split(std::back_insert_iterator<std::vector<std::string> > out,
		std::string& str, RTC_regex& preg)
{

  int result,i;
  regmatch_t pmatch[1];

  result = regexec(&preg._pattern, str.c_str(), 1, pmatch, 0);

  if(result == 1){
	  //out.push_back(str);
	  out = str;
	  return false;
  }

//  out.push_back( str.substr(0, pmatch[0].rm_so) );
  out = str.substr(0, pmatch[0].rm_so) ;
  std::string v(str.substr(pmatch[0].rm_eo));

  regex_split(out, v, preg);

  return true;
}
/*
 *
 *  itoa: Integer to String
 */

int itoa(int n, char *str, int dx){
  int i;
  int len = 1;
  int tmp = n;
  int top = 0;

  if (n < 0){
    tmp = -tmp;
    len++;
    str[0] = '-';
    top = 1;
  }

  while((tmp = tmp / dx) != 0) len++;

  if (sizeof(str) < len) return 1;

  str[len] = '\0';
  for(i=len - 1;i >= top;i--){
    str[i] = '0' + n % dx;
    n = n / dx;
  }

  return 0;
}
