/*
 * Functor library for OpenRTM
 *     
 */
#include <functional>

namespace std
{
  //
  // defun ternary_function
  //
  template <class _Arg1, class _Arg2, class _Arg3, class _Result>
  struct ternary_function{
	typedef _Arg1 first_argument_type;
	typedef _Arg2 second_argument_type;
	typedef _Arg3 third_argument_type;
	typedef _Result result_type;
  };

  //
  // defun pointer_to_ternary_function
  //
  template <class _Arg1, class _Arg2, class _Arg3, class _Result>
  class pointer_to_ternary_function :
    public ternary_function<_Arg1, _Arg2, _Arg3, _Result>{
  protected:
    _Result (*_M_ptr)(_Arg1,_Arg2,_Arg3);
  public:
   pointer_to_ternary_function(){}
   explicit pointer_to_ternary_function(_Result (*__x)(_Arg1,_Arg2,_Arg3))
	 : _M_ptr(__x){}
   _Result operator()(_Arg1,_Arg2,_Arg3){
	   return _M_ptr(__x,__y,__z);
   }
  };
  
  template <class _Arg1, class _Arg2, class _Arg3, class _Result>
  inline pointer_to_ternary_function <_Arg1, _Arg2, _Arg3, _Result>
    ptr_fun(_Result (*__x)(_Arg1, _Arg2, _Arg3)){
     return pointer_to_ternary_function <_Arg1, _Arg2, _Arg3, _Result>(__x);
  }
  
 // 
 // mem_fun2_t and mem_fun2_ref_t
 //
  template<class _Ret, class _Tp, class _Arg1, class _Arg2>
  class mem_fun2_t : public ternary_function<_Tp*,_Arg1,_Arg2,_Ret>{
  public:
    explicit mem_fun2_t(_Ret (_Tp::*__pf)(_Arg1, _Arg2)) : _M_f(__pf){}
    _Ret operator()(_Tp* __p,_Arg1 __x, _Arg2 __y) {
	    return (__p->*_M_f)(__x, __y); }
  private:
    _Ret (_Tp::*_M_f)(_Arg1,_Arg2);
  };
  
  template<class _Ret, class _Tp, class _Arg1, class _Arg2>
  class mem_fun2_ref_t : public ternary_function<_Tp*,_Arg1,_Arg2,_Ret>{
    public:
  explicit mem_fun2_ref_t(_Ret (_Tp::*__pf)(_Arg1, _Arg2)) : _M_f(__pf){}
  _Ret operator()(_Tp& __r, _Arg1 __x, _Arg2 __y) {
	  return (__r.*_M_f)(__x, __y); }
  private:
    _Ret (_Tp::*_M_f)(_Arg1,_Arg2);
  };
  
  /*
   * Helper functions for mem_fun2
   */
  template<class _Ret, class _Tp, class _Arg1, class _Arg2>
  inline mem_fun2_t<_Ret,_Tp, _Arg1,_Arg2> 
          mem_fun(_Ret (_Tp::*__f)(_Arg1,_Arg2)) {
		  return mem_fun2_t<_Ret,_Tp,_Arg1,_Arg2>(__f); }
  
  template<class _Ret, class _Tp, class _Arg1, class _Arg2>
  inline mem_fun2_ref_t<_Ret,_Tp, _Arg1,_Arg2> 
         mem_fun_ref(_Ret (_Tp::*__f)(_Arg1,_Arg2)) {
		 return mem_fun2_ref_t<_Ret,_Tp,_Arg1,_Arg2>(__f); }

  /**
   * Simple binder 
   */
  template <class _Operation>
  class binder1st_2
    : public binary_function<typename _Operation::second_argument_type,
    	typename _Operation::third_argument_type,
 	typename _Operation::result_type> {
  protected:
     _Operation op;
    typename _Operation::first_argument_type value;
  public:
    binder1st_2(const _Operation& __x,
                const typename _Operation::first_argument_type& __y)
            : op(__x), value(__y) {}
    typename _Operation::result_type
         operator()(const typename _Operation::second_argument_type& __x,
                    typename _Operation::third_argument_type& __y)  {
    return op(value, __x,__y);
  }
};
  
  template <class _Operation, class _Tp>
  inline binder1st_2<_Operation>
      bind1st_2( const _Operation& __fn, const _Tp& __x)
  {
    typedef typename _Operation::first_argument_type _Arg1_type;
    return binder1st_2<_Operation>(__fn, _Arg1_type(__x));
  };
  
}

